@@ -58,6 +58,7 @@
 
 struct _PanelAppletFramePrivate {
 	GNOME_Vertigo_PanelAppletShell  applet_shell;
+	CORBA_Object                    control;
 	Bonobo_PropertyBag              property_bag;
 	BonoboUIComponent              *ui_component;
 
@@ -85,6 +86,9 @@ typedef enum {
 	APPLET_HAS_HANDLE   = 1 << 2
 } PanelAppletFlags;
 
+static void panel_applet_frame_cnx_broken (ORBitConnection  *cnx,
+					   PanelAppletFrame *frame);
+
 static void
 panel_applet_frame_sync_menu_state (PanelAppletFrame *frame)
 {
@@ -99,7 +103,7 @@ panel_applet_frame_sync_menu_state (PanelAppletFrame *frame)
 
 	lockable = panel_applet_lockable (frame->priv->applet_info);
 	movable = panel_applet_can_freely_move (frame->priv->applet_info);
-	removable = panel_profile_list_is_writable (PANEL_GCONF_APPLETS);
+	removable = panel_profile_id_lists_are_writable ();
 
 	locked = panel_widget_get_applet_locked (panel_widget, GTK_WIDGET (frame));
 
@@ -349,8 +353,9 @@ panel_applet_frame_create (PanelToplevel *toplevel,
 	key = panel_gconf_full_key (PANEL_GCONF_APPLETS, profile, id, "bonobo_iid");
 	gconf_client_set_string (client, key, iid, NULL);
 
-	/* frees id */
 	panel_profile_add_to_list (PANEL_GCONF_APPLETS, id);
+
+	g_free (id);
 }
 
 void
@@ -469,6 +474,13 @@ panel_applet_frame_finalize (GObject *object)
 		bonobo_object_unref (
 			BONOBO_OBJECT (frame->priv->ui_component));
 
+	if (frame->priv->control) {
+		ORBit_small_unlisten_for_broken (frame->priv->control,
+						 G_CALLBACK (panel_applet_frame_cnx_broken));
+		CORBA_Object_release (frame->priv->control, NULL);
+		frame->priv->control = CORBA_OBJECT_NIL;
+	}
+
 	g_free (frame->priv->iid);
 	frame->priv->iid = NULL;
 
@@ -799,7 +811,7 @@ panel_applet_frame_reload_response (GtkWidget        *dialog,
 		/* if we can't write to applets list we can't really delete
 		   it, so we'll just ignore this.  FIXME: handle this
 		   more correctly I suppose. */
-		if (panel_profile_list_is_writable (PANEL_GCONF_APPLETS))
+		if (panel_profile_id_lists_are_writable ())
 			panel_profile_delete_object (info);
 	}
 
@@ -844,7 +856,8 @@ panel_applet_frame_get_name (char *iid)
 }
 
 static void
-panel_applet_frame_cnx_broken (PanelAppletFrame *frame)
+panel_applet_frame_cnx_broken (ORBitConnection  *cnx,
+			       PanelAppletFrame *frame)
 {
 	GtkWidget *dialog;
 	GdkScreen *screen;
@@ -1002,7 +1015,7 @@ panel_applet_frame_loading_failed (PanelAppletFrame  *frame,
 
 	if (response == LOADING_FAILED_RESPONSE_DELETE &&
 	    !locked_down &&
-	    panel_profile_list_is_writable (PANEL_GCONF_APPLETS))
+	    panel_profile_id_lists_are_writable ())
 		panel_profile_remove_from_list (PANEL_GCONF_APPLETS, id);
 }
 
@@ -1196,12 +1209,11 @@ panel_applet_frame_construct (PanelAppletFrame *frame,
 			      const char       *iid,
 			      const char       *id)
 {
-	BonoboControlFrame    *control_frame;
-	Bonobo_Control         control;
-	CORBA_Environment      ev;
-	ORBitConnectionStatus  cnx_status;
-	GtkWidget             *widget;
-	char                  *moniker;
+	BonoboControlFrame *control_frame;
+	Bonobo_Control      control;
+	CORBA_Environment   ev;
+	GtkWidget          *widget;
+	char               *moniker;
 
 	frame->priv->panel = panel;
 
@@ -1229,6 +1241,11 @@ panel_applet_frame_construct (PanelAppletFrame *frame,
 
 	frame->priv->iid = g_strdup (iid);
 
+	frame->priv->control = CORBA_Object_duplicate (control, NULL);
+	ORBit_small_listen_for_broken (control,
+				       G_CALLBACK (panel_applet_frame_cnx_broken),
+				       frame);
+
 	widget = bonobo_widget_new_control_from_objref (control, CORBA_OBJECT_NIL);
 
 	bonobo_object_release_unref (control, NULL);
@@ -1307,27 +1324,6 @@ panel_applet_frame_construct (PanelAppletFrame *frame,
 
 	CORBA_exception_free (&ev);
 
-	cnx_status = ORBit_small_get_connection_status (control);
-	if (cnx_status != ORBIT_CONNECTION_IN_PROC) {
-		ORBitConnection *cnx = ORBit_small_get_connection (control);
-
-		if (cnx == NULL) {
-			CORBA_exception_free (&ev);
-			panel_applet_frame_loading_failed (
-				frame, NULL, iid, id, GTK_WINDOW (panel->toplevel));
-			g_warning (G_STRLOC ": failed to load applet %s (lost connection)", iid);
-			gtk_object_sink (GTK_OBJECT (widget));
-			return NULL;
-		}
-
-		g_signal_connect_object (
-			cnx,
-			"broken",
-			G_CALLBACK (panel_applet_frame_cnx_broken),
-			frame,
-			G_CONNECT_SWAPPED);
-	}
-	
 	gtk_container_add (GTK_CONTAINER (frame), widget);
 
 	return widget;
